{% extends "django_dowser/base.html" %}

{% block title %}Dowser: Types{% endblock title %}

{% block body %}
<div id="header">
    <h1><a href="{% url 'dowser_index' %}">Dowser</a>: Types</h1>
</div>

<div id="output">
{{output|safe}}
</div>

{% endblock body %}
