<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <title>{% block title %}Dowser{% endblock title %}</title>
<style type='text/css'>
{% include "django_dowser/main.css" %}
</style>
</head>

<body>
{% block body %}
<div id="header">
    <h1><a href="{% url 'dowser_index' %}">Dowser</a></h1>
</div>
{% endblock body %}
</body>
</html>
