body {
    margin: 0;
    padding: 0;
}

#header {
    text-align: center;
    background-color: #CCCCCC;
    margin: 0;
    padding: 0.25em;
}

#header p {
    margin: 0;
    padding: 0.25em;
}

#header h1 a {
    text-decoration: none;
}

#header h1 a:visited {
    color: black;
}

h1 {
    font: 900 20pt Verdana, sans-serif;
    text-align: center;
    margin: 0;
    padding: 0.25em;
}

h2 {
    text-align: center;
}

.obj {
    border: 1px dashed #CCCCCC;
    padding: 0.5em;
    margin: 2px;
    font: 10pt Arial, sans-serif;
    vertical-align: middle;
}

.typename {
    font-weight: bold;
}

.refs {
    border: 1px solid #CCCCCC;
    padding: 0.25em;
    margin: 0;
}



.typecount {
    border: 1px dashed #CCCCCC;
    padding: 0.5em;
    margin: 2px;
    font: 10pt Arial, sans-serif;
    vertical-align: middle;
    float: left;
    width: 30%;
}

.chart {
    border-bottom: 1px solid #CCCCCC;
    background-color: white;
    padding: 0.25em;
    margin: 0;
}

.gparent {
    border: 1px dashed #CCCCCC;
    padding: 0.5em;
    margin: 2px 2px 2px 2em;
    font: 10pt Arial, sans-serif;
    vertical-align: middle;
}

.attr {
    margin: 0.25em;
    padding: 0;
    font: 9pt Courier, monospace;
}

.branch, .seen {
    padding: 1px;
    margin: 0 0 0 1em;
    border: 1px solid #CCCCCC;
}

